package handler

import (
	"fmt"
	"log"
	"my-blog/article"
	"net/http"
	"strconv"
	"unicode/utf8"
)

func (h *Handler) Article(res http.ResponseWriter, req *http.Request) {
	switch req.Method {
	case http.MethodGet:
		h.getArticle(res, req)
	case http.MethodPost:
		h.createArticle(res, req)
	case http.MethodDelete:
		h.deleteArticle(res, req)
	default:
		res.WriteHeader(http.StatusMethodNotAllowed)
	}
}

func (h *Handler) getArticle(res http.ResponseWriter, req *http.Request) {
	queryID := req.URL.Query().Get("id")
	id, err := strconv.Atoi(queryID)
	if err != nil {
		log.Println("failed to parse query:", err)
		res.WriteHeader(http.StatusBadRequest)
		return
	}

	a, err := h.article.Get(id)
	if err != nil {
		log.Println("failed to get article:", err)
		res.WriteHeader(http.StatusNotFound)
		return
	}

	params := struct {
		Title   string
		Article *article.Article
	}{
		Title:   "わたしのブログ",
		Article: a,
	}
	h.templateArticle.Execute(res, params)
}

func (h *Handler) createArticle(res http.ResponseWriter, req *http.Request) {
	title := req.PostFormValue("title")
	content := req.PostFormValue("content")

	// 長さチェック
	if err := validate(title, content); err != nil {
		res.WriteHeader(http.StatusBadRequest)
		res.Write([]byte(err.Error()))
		return
	}

	id, err := h.article.Create(title, content)
	if err != nil {
		log.Println("failed to create article:", err)
		res.WriteHeader(http.StatusInternalServerError)
		return
	}

	res.Header().Add("Location", fmt.Sprintf("/articles?id=%d", id))
	res.WriteHeader(http.StatusSeeOther)
}

func validate(title, content string) error {
	if len(title) == 0 {
		return fmt.Errorf("タイトルを入力してください。")
	}
	if len(content) == 0 {
		return fmt.Errorf("内容を入力してください。")
	}
	if utf8.RuneCountInString(title) > 100 {
		return fmt.Errorf("タイトルは100文字以内に収めてください。")
	}
	if utf8.RuneCountInString(content) > 1000 {
		return fmt.Errorf("内容は1000文字以内に収めてください。")
	}
	return nil
}

func (h *Handler) deleteArticle(res http.ResponseWriter, req *http.Request) {
	// IDを取得
	queryID := req.URL.Query().Get("id")
	id, err := strconv.Atoi(queryID)
	if err != nil {
		log.Println("failed to parse query:", err)
		res.WriteHeader(http.StatusBadRequest)
		return
	}

	// 削除を実行
	if err := h.article.Delete(id); err != nil {
		log.Println("failed to delete article:", err)
		res.WriteHeader(http.StatusInternalServerError)
		return
	}

	res.WriteHeader(http.StatusNoContent)
}

func (h *Handler) NewArticle(res http.ResponseWriter, req *http.Request) {
	params := struct {
		Title string
	}{
		Title: "わたしのブログ",
	}
	h.templateNewArticle.Execute(res, params)
}
